subroutine CalcEnergy(Pos, M, L, rc, eps, lambda, Name, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NAtom-1) :: Name
    real(8), intent(in) :: L, rc, eps, lambda
    real(8), intent(out) :: PEnergy
    real(8), dimension(Dim) :: rij, Posi
    real(8), parameter :: k = 3000., r0 = 1.
    real(8) :: d, d2, id2, id6, id12, Shift, rc2, ShiftColl, charge
    integer :: i, j
    !======== YOUR CODE HERE ========
    PEnergy = 0.
    rc2 = rc * rc
    Shift = (-4. * (rc**(-12) - rc**(-6)))
    ShiftColl= (-4. * ((rc/5.)**(-12) - (rc/5.)**(-6)))

    do i = 0, NAtom - 1
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
            if (j == i + 1 .and. mod((j-2), M) > 0 .and. Name(i)==1) then  
                d = sqrt(d2)
                PEnergy = PEnergy + (k) / (2.) * (d - r0) * (d - r0)
                        !this is a bonded interaction;
                        !use the harmonic bond potential here
            else
                if (d2 <= rc2) then
                    if (Name(i)==Name(j)) then 
                        if (Name(i)==1) then
                            id2 = 1. / d2            !inverse squared distance
                            id6 = id2 * id2 * id2    !inverse sixth distance
                            id12 = id6 * id6         !inverse twelvth distance
                            PEnergy = PEnergy + 4. * (id12 - id6) + Shift
                        endif
                    else
                        id2 = 25. / (d2)            !inverse squared distance
                        id6 = id2 * id2 * id2    !inverse sixth distance
                        id12 = id6 * id6         !inverse twelvth distance
                        PEnergy = PEnergy + 4. * (id12 - id6) + ShiftColl
                    endif
                endif
                charge=-10.
                if (Name(i)==Name(j)) then
                    if (Name(i)==0) then
                        charge = 100.
                    else
                        charge = 1.
                    endif
                endif                             
                d = sqrt(d2)
                PEnergy = PEnergy + charge*eps/d*exp(-1.0*d*lambda)
                !this is a nonbonded interaction;
                !use the Lennard-Jones potential here

            endif
        enddo
    enddo
end subroutine

subroutine CalcEnergyForces(Pos, M, L, rc, eps, lambda, Name, PEnergy, Forces, CollForces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NAtom-1) :: Name
    real(8), intent(in) :: L, rc, eps, lambda
    real(8), intent(out) :: PEnergy
    real(8), intent(out), dimension(0:1,0:Dim-1) :: CollForces
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), dimension(Dim) :: rij, Posi, Fij
    real(8), parameter :: k = 3000., r0 = 1.
    real(8) :: d, d2, id2, id6, id12, Shift, rc2, ShiftColl, charge
    integer :: i, j
    !======== YOUR CODE HERE ========
    PEnergy = 0.
    Forces = 0.
    CollForces=0
    rc2 = rc * rc
    Shift = -4. * (rc**(-12) - rc**(-6))
    ShiftColl= -4. * ((rc/5.)**(-12) - (rc/5.)**(-6))

    do i = 0, NAtom - 1
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            d2 = sum(rij * rij)
            if (j == i + 1 .and. mod((j-2), M) > 0 .and. Name(i)==1) then
                    d = sqrt(d2)
                    PEnergy = PEnergy + (k) / (2.) * (d - r0) * (d - r0)
                    Fij = rij * k * (1 - r0 / d)
                    Forces(i,:) = Forces(i,:) + Fij
                    Forces(j,:) = Forces(j,:) - Fij
                !this is a bonded interaction;
                !use the harmonic bond potential here
            else
                if (d2 <= rc2) then
                    if (Name(i)==Name(j)) then 
                        if (Name(i)==1) then
                            id2 = 1. / d2            !inverse squared distance
                            id6 = id2 * id2 * id2    !inverse sixth distance
                            id12 = id6 * id6         !inverse twelvth distance
                            PEnergy = PEnergy + 4. * (id12 - id6) + Shift
                            Fij = rij * (-48. * id12 + 24. * id6) * id2
                            Forces(i,:) = Forces(i,:) + Fij
                            Forces(j,:) = Forces(j,:) - Fij
                        endif
                    else
                        id2 = 25. / (d2)            !inverse squared distance
                        id6 = id2 * id2 * id2    !inverse sixth distance
                        id12 = id6 * id6         !inverse twelvth distance
                        PEnergy = PEnergy + 4. * (id12 - id6) + ShiftColl
                        Fij = rij * ((-48. * id12 + 24. * id6) * (id2/25.))
                        Forces(i,:) = Forces(i,:) + Fij
                        Forces(j,:) = Forces(j,:) - Fij
                    endif
                endif
                charge=-10.
                if (Name(i)==Name(j)) then
                    if (Name(i)==0) then
                        charge = 100.
                    else
                        charge = 1.
                    endif
                endif                  
                d = sqrt(d2)
                PEnergy = PEnergy + charge*eps/d*exp(-1.0*d*lambda)
                Fij = rij * exp((-1.0)*d*lambda) * (-1.0*charge*eps) * (1.0/(d*d*d)+lambda/(d*d))
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
                !this is a nonbonded interaction;
                !use the Lennard-Jones potential here

            endif
        enddo
    enddo
    CollForces=Forces(0:1,:)
    Forces(0,:)=[0.,0.,0.]
    Forces(1,:)=[0.,0.,0.]

end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, M, L, rc, dt, eps, lambda, Name, KEnergy, PEnergy, CollForces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt, eps, lambda
    real(8), intent(in), dimension(0:NAtom-1) :: Name
    real(8), intent(out), dimension(0:1,0:Dim-1) :: CollForces
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, L, rc, eps, lambda, Name, PEnergy, Accel, CollForces, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine